@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef PROFILE_H
 #define PROFILE_H
 
@@ -20,14 +21,12 @@ struct divecomputer;
 struct plot_info;
 struct plot_data {
 	unsigned int in_deco : 1;
-	int cylinderindex;
 	int sec;
-	/* pressure[0] is sensor cylinder pressure [when CCR, the pressure of the diluent cylinder]
-	 * pressure[1] is interpolated cylinder pressure */
-	int pressure[2];
-	/* o2pressure[0] is o2 cylinder pressure [CCR]
-	 * o2pressure[1] is interpolated o2 cylinder pressure [CCR] */
-	int o2cylinderpressure[2];
+	/*
+	 * pressure[x][0] is sensor pressure for cylinder x
+	 * pressure[x][1] is interpolated pressure
+	 */
+	int pressure[MAX_CYLINDERS][2];
 	int temperature;
 	/* Depth info */
 	int depth;
@@ -63,6 +62,7 @@ struct plot_data {
 	int bearing;
 	double ambpressure;
 	double gfline;
+	double density;
 };
 
 struct ev_select {
@@ -95,13 +95,9 @@ int get_maxdepth(struct plot_info *pi);
 
 #define SENSOR_PR 0
 #define INTERPOLATED_PR 1
-#define SENSOR_PRESSURE(_entry) (_entry)->pressure[SENSOR_PR]
-#define O2CYLINDER_PRESSURE(_entry) (_entry)->o2cylinderpressure[SENSOR_PR]
-#define CYLINDER_PRESSURE(_o2, _entry) (_o2 ? O2CYLINDER_PRESSURE(_entry) : SENSOR_PRESSURE(_entry))
-#define INTERPOLATED_PRESSURE(_entry) (_entry)->pressure[INTERPOLATED_PR]
-#define INTERPOLATED_O2CYLINDER_PRESSURE(_entry) (_entry)->o2cylinderpressure[INTERPOLATED_PR]
-#define GET_PRESSURE(_entry) (SENSOR_PRESSURE(_entry) ? SENSOR_PRESSURE(_entry) : INTERPOLATED_PRESSURE(_entry))
-#define GET_O2CYLINDER_PRESSURE(_entry) (O2CYLINDER_PRESSURE(_entry) ? O2CYLINDER_PRESSURE(_entry) : INTERPOLATED_O2CYLINDER_PRESSURE(_entry))
+#define SENSOR_PRESSURE(_entry,_idx) (_entry)->pressure[_idx][SENSOR_PR]
+#define INTERPOLATED_PRESSURE(_entry,_idx) (_entry)->pressure[_idx][INTERPOLATED_PR]
+#define GET_PRESSURE(_entry,_idx) (SENSOR_PRESSURE(_entry,_idx) ? SENSOR_PRESSURE(_entry,_idx) : INTERPOLATED_PRESSURE(_entry,_idx))
 #define SAC_WINDOW 45 /* sliding window in seconds for current SAC calculation */
 
 #ifdef __cplusplus
