@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "DiveObjectHelper.h"
 
 #include <QDateTime>
@@ -84,7 +85,7 @@ QString DiveObjectHelper::date() const
 {
 	QDateTime localTime = QDateTime::fromMSecsSinceEpoch(1000*m_dive->when, Qt::UTC);
 	localTime.setTimeSpec(Qt::UTC);
-	return localTime.date().toString(prefs.date_format);
+	return localTime.date().toString(prefs.date_format_short);
 }
 
 timestamp_t DiveObjectHelper::timestamp() const
@@ -123,7 +124,7 @@ QString DiveObjectHelper::gps_decimal() const
 
 QString DiveObjectHelper::duration() const
 {
-	return get_dive_duration_string(m_dive->duration.seconds, QObject::tr("h:"), QObject::tr("min"));
+	return get_dive_duration_string(m_dive->duration.seconds, QObject::tr("h"), QObject::tr("min"));
 }
 
 bool DiveObjectHelper::noDive() const
@@ -332,12 +333,14 @@ QString DiveObjectHelper::tripMeta() const
 	QString ret = EMPTY_DIVE_STRING;
 	struct dive_trip *dt = m_dive->divetrip;
 	if (dt) {
-		QString numDives = tr("%1 dive(s)").arg(dt->nrdives);
+		QString numDives = tr("(%n dive(s))", "", dt->nrdives);
 		QString title(dt->location);
+		QDateTime firstTime = QDateTime::fromMSecsSinceEpoch(1000*dt->when, Qt::UTC);
+		QString firstMonth = firstTime.toString("MMM");
+		QString tripDate = QStringLiteral("%1@%2").arg(firstMonth,firstTime.toString("yy"));
+;
 		if (title.isEmpty()) {
 			// so use the date range
-			QDateTime firstTime = QDateTime::fromMSecsSinceEpoch(1000*dt->when, Qt::UTC);
-			QString firstMonth = firstTime.toString("MMM");
 			QString firstYear = firstTime.toString("yyyy");
 			QDateTime lastTime = QDateTime::fromMSecsSinceEpoch(1000*dt->dives->when, Qt::UTC);
 			QString lastMonth = lastTime.toString("MMM");
@@ -349,7 +352,7 @@ QString DiveObjectHelper::tripMeta() const
 			else
 				title = firstMonth + " " + firstYear + " - " + lastMonth + " " + lastYear;
 		}
-		ret = QString::number((quint64)m_dive->divetrip, 16) + QLatin1Literal("::") + QStringLiteral("%1 (%2)").arg(title, numDives);
+		ret = QString::number((quint64)m_dive->divetrip, 16) + QLatin1Literal("++") +  tripDate + QLatin1Literal("::") + QStringLiteral("%1 %2").arg(title, numDives);
 	}
 	return ret;
 }
