@@ -150,8 +150,12 @@ bool imported = false;
 
 static void print_version()
 {
-	printf("Subsurface v%s, ", subsurface_git_version());
+	printf("Subsurface v%s,\n", subsurface_git_version());
 	printf("built with libdivecomputer v%s\n", dc_version(NULL));
+	print_qt_versions();
+	int git_maj, git_min, git_rev;
+	git_libgit2_version(&git_maj, &git_min, &git_rev);
+	printf("built with libgit2 %d.%d.%d\n", git_maj, git_min, git_rev);
 }
 
 void print_files()
@@ -197,8 +201,6 @@ static void print_help()
 	printf("\n --survey              Offer to submit a user survey");
 	printf("\n --user=<test>         Choose configuration space for user <test>");
 	printf("\n --cloud-timeout=<nr>  Set timeout for cloud connection (0 < timeout < 60)");
-	printf("\n --win32console        Create a dedicated console if needed (Windows only). Add option before everything else");
-	printf("\n --win32log            Write the program output to subsurface.log (Windows only). Add option before everything else\n\n");
 }
 
 void parse_argument(const char *arg)
@@ -211,6 +213,7 @@ void parse_argument(const char *arg)
 			print_help();
 			exit(0);
 		case 'v':
+			print_version();
 			verbose++;
 			continue;
 		case 'q':
@@ -239,6 +242,7 @@ void parse_argument(const char *arg)
 				return;
 			}
 			if (strcmp(arg, "--verbose") == 0) {
+				print_version();
 				verbose++;
 				return;
 			}
@@ -254,10 +258,6 @@ void parse_argument(const char *arg)
 				++force_root;
 				return;
 			}
-			if (strcmp(arg, "--win32console") == 0)
-				return;
-			if (strcmp(arg, "--win32log") == 0)
-				return;
 		/* fallthrough */
 		case 'p':
 			/* ignore process serial number argument when run as native macosx app */
