@@ -7,7 +7,7 @@
 #include "mapwidget.h"
 #include "core/dive.h"
 #include "core/divesite.h"
-#include "mobile-widgets/qmlmapwidgethelper.h"
+#include "map-widget/qmlmapwidgethelper.h"
 #include "qt-models/maplocationmodel.h"
 #include "mainwindow.h"
 #include "divelistview.h"
@@ -118,6 +118,12 @@ void MapWidget::coordinatesChangedLocal()
 	emit coordinatesChanged();
 }
 
+void MapWidget::updateCurrentDiveSiteCoordinatesToMap()
+{
+	CHECK_IS_READY_RETURN_VOID();
+	m_mapHelper->updateCurrentDiveSiteCoordinatesToMap();
+}
+
 MapWidget::~MapWidget()
 {
 	m_instance = NULL;
