@@ -22,6 +22,18 @@ PreferencesLanguage::PreferencesLanguage() : AbstractPreferencesWidget(tr("Langu
 	filterModel->sort(0);
 	connect(ui->languageFilter, &QLineEdit::textChanged,
 			filterModel, &QSortFilterProxyModel::setFilterFixedString);
+
+	dateFormatShortMap.insert("MM/dd/yyyy", "M/d/yy");
+	dateFormatShortMap.insert("dd.MM.yyyy", "d.M.yy");
+	dateFormatShortMap.insert("yyyy-MM-dd", "yy-M-d");
+	foreach (QString format, dateFormatShortMap.keys())
+		ui->dateFormatEntry->addItem(format);
+	connect(ui->dateFormatEntry, SIGNAL(currentIndexChanged(const QString&)),
+		this, SLOT(dateFormatChanged(const QString&)));
+
+	ui->timeFormatEntry->addItem("hh:mm");
+	ui->timeFormatEntry->addItem("h:mm AP");
+	ui->timeFormatEntry->addItem("hh:mm AP");
 }
 
 PreferencesLanguage::~PreferencesLanguage()
@@ -29,13 +41,18 @@ PreferencesLanguage::~PreferencesLanguage()
 	delete ui;
 }
 
+void PreferencesLanguage::dateFormatChanged(const QString &text)
+{
+	ui->shortDateFormatEntry->setText(dateFormatShortMap.value(text));
+}
+
 void PreferencesLanguage::refreshSettings()
 {
 	ui->languageSystemDefault->setChecked(prefs.locale.use_system_language);
 	ui->timeFormatSystemDefault->setChecked(!prefs.time_format_override);
 	ui->dateFormatSystemDefault->setChecked(!prefs.date_format_override);
-	ui->timeFormatEntry->setText(prefs.time_format);
-	ui->dateFormatEntry->setText(prefs.date_format);
+	ui->timeFormatEntry->setCurrentText(prefs.time_format);
+	ui->dateFormatEntry->setCurrentText(prefs.date_format);
 	ui->shortDateFormatEntry->setText(prefs.date_format_short);
 	QAbstractItemModel *m = ui->languageDropdown->model();
 	QModelIndexList languages = m->match(m->index(0, 0), Qt::UserRole, QString(prefs.locale.lang_locale).replace("-", "_"));
@@ -71,19 +88,20 @@ void PreferencesLanguage::syncSettings()
 	lang->setUseSystemLanguage(ui->languageSystemDefault->isChecked());
 	lang->setTimeFormatOverride(!ui->timeFormatSystemDefault->isChecked());
 	lang->setDateFormatOverride(!ui->dateFormatSystemDefault->isChecked());
-	lang->setTimeFormat(ui->timeFormatEntry->text());
-	lang->setDateFormat(ui->dateFormatEntry->text());
+	lang->setTimeFormat(ui->timeFormatEntry->currentText());
+	lang->setDateFormat(ui->dateFormatEntry->currentText());
 	lang->setDateFormatShort(ui->shortDateFormatEntry->text());
 	uiLanguage(NULL);
 
+	QString qDateTimeWeb = tr("These will be used as is. This might not be what you intended.\nSee http://doc.qt.io/qt-5/qdatetime.html#toString");
 	QRegExp tfillegalchars("[^hHmszaApPt\\s:;\\.,]");
-	if (tfillegalchars.indexIn(ui->timeFormatEntry->text()) >= 0)
+	if (tfillegalchars.indexIn(ui->timeFormatEntry->currentText()) >= 0)
 		QMessageBox::warning(this, tr("Literal characters"),
-			tr("Non-special character(s) in time format.\nThese will be used as is. This might not be what you intended.\nSee http://doc.qt.io/qt-5/qdatetime.html#toString"));
+			tr("Non-special character(s) in time format.\n") + qDateTimeWeb);
 
-	QRegExp dfillegalchars("[^dMy/\\s:;\\.,]");
-	if (dfillegalchars.indexIn(ui->dateFormatEntry->text()) >= 0 ||
+	QRegExp dfillegalchars("[^dMy/\\s:;\\.,\\-]");
+	if (dfillegalchars.indexIn(ui->dateFormatEntry->currentText()) >= 0 ||
 	    dfillegalchars.indexIn(ui->shortDateFormatEntry->text()) >= 0)
 		QMessageBox::warning(this, tr("Literal characters"),
-				     tr("Non-special character(s) in time format.\nThese will be used as is. This might not be what you intended.\nSee http://doc.qt.io/qt-5/qdatetime.html#toString"));
+			tr("Non-special character(s) in date format.\n") + qDateTimeWeb);
 }
