@@ -129,7 +129,7 @@ void PrintOptions::on_editButton_clicked()
 void PrintOptions::on_importButton_clicked()
 {
 	QString filename = QFileDialog::getOpenFileName(this, tr("Import template file"), "",
-							tr("HTML files (*.html)"));
+							tr("HTML files") + " (*.html)");
 	if (filename.isEmpty())
 		return;
 	QFileInfo fileInfo(filename);
@@ -142,7 +142,7 @@ void PrintOptions::on_importButton_clicked()
 void PrintOptions::on_exportButton_clicked()
 {
 	QString filename = QFileDialog::getSaveFileName(this, tr("Export template files as"), "",
-							tr("HTML files (*.html)"));
+							tr("HTML files") + " (*.html)");
 	if (filename.isEmpty())
 		return;
 	QFile::copy(getPrintingTemplatePathUser() + QDir::separator() + getSelectedTemplate(), filename);
