@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "qmlmanager.h"
 #include <QUrl>
 #include <QSettings>
@@ -10,6 +11,11 @@
 #include <QApplication>
 #include <QElapsedTimer>
 #include <QTimer>
+#include <QDateTime>
+
+#if defined(BT_SUPPORT)
+#include <QBluetoothLocalDevice>
+#endif
 
 #include "qt-models/divelistmodel.h"
 #include "qt-models/gpslistmodel.h"
@@ -23,6 +29,7 @@
 #include "core/subsurface-qt/SettingsObjectWrapper.h"
 #include "core/membuffer.h"
 #include "qt-models/tankinfomodel.h"
+#include "core/downloadfromdcthread.h"
 
 QMLManager *QMLManager::m_instance = NULL;
 
@@ -31,6 +38,15 @@ QMLManager *QMLManager::m_instance = NULL;
 
 #define NOCLOUD_LOCALSTORAGE format_string("%s/cloudstorage/localrepo[master]", system_default_directory())
 
+static void progressCallback(const char *text)
+{
+	QMLManager *self = QMLManager::instance();
+	if (self) {
+		self->appendTextToLog(QString(text));
+		self->setProgressMessage(QString(text));
+	}
+}
+
 static void appendTextToLogStandalone(const char *text)
 {
 	QMLManager *self = QMLManager::instance();
@@ -38,45 +54,52 @@ static void appendTextToLogStandalone(const char *text)
 		self->appendTextToLog(QString(text));
 }
 
-extern "C" int gitProgressCB(bool reset, const char *text)
+// show the git progress in the passive notification area
+extern "C" int gitProgressCB(const char *text)
 {
 	static QElapsedTimer timer;
-	static qint64 lastTime;
-	static QString lastText;
+	static qint64 lastTime = 0;
 	static QMLManager *self;
-	static int lastPercent;
 
 	if (!self)
 		self = QMLManager::instance();
 
-	if (!timer.isValid() || reset) {
+	if (!timer.isValid()) {
 		timer.restart();
 		lastTime = 0;
-		lastPercent = prefs.git_local_only ? -2 : 0;
-		lastText.clear();
 	}
 	if (self) {
 		qint64 elapsed = timer.elapsed();
-		// don't show the same status twice in 200ms
-		if (lastText == text && elapsed - lastTime < 200)
-			return 0;
-		if (lastPercent < 0)
-			lastPercent--;
-		else
-			lastPercent++;
-		self->loadDiveProgress(lastPercent);
-		QString logText = QString::number(elapsed / 1000.0, 'f', 1) + " / " + QString::number((elapsed - lastTime) / 1000.0, 'f', 3) +
-				  QString(" : git %1 (%2)").arg(lastPercent).arg(text);
-		self->appendTextToLog(logText);
-		qDebug() << logText;
-		if (elapsed - lastTime > 500)
-			qApp->processEvents();
+		self->appendTextToLog(text);
+		self->setNotificationText(text);
+		if (elapsed - lastTime > 50) { // 20 Hz refresh
+			qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+		}
 		lastTime = elapsed;
 	}
 	// return 0 so that we don't end the download
 	return 0;
 }
 
+#if defined(BT_SUPPORT)
+void QMLManager::btHostModeChange(QBluetoothLocalDevice::HostMode state)
+{
+	BTDiscovery *btDiscovery = BTDiscovery::instance();
+
+	qDebug() << "btHostModeChange to " << state;
+	if (state != QBluetoothLocalDevice::HostPoweredOff) {
+		connectionListModel.removeAllAddresses();
+		btDiscovery->BTDiscoveryReDiscover();
+		m_btEnabled = btDiscovery->btAvailable();
+	} else {
+		connectionListModel.removeAllAddresses();
+		set_non_bt_addresses();
+		m_btEnabled = false;
+	}
+	emit btEnabledChanged();
+}
+#endif
+
 QMLManager::QMLManager() : m_locationServiceEnabled(false),
 	m_verboseEnabled(false),
 	reply(0),
@@ -84,21 +107,48 @@ QMLManager::QMLManager() : m_locationServiceEnabled(false),
 	deletedTrip(0),
 	m_updateSelectedDive(-1),
 	m_selectedDiveTimestamp(0),
-	m_credentialStatus(UNKNOWN),
-	alreadySaving(false)
+	m_credentialStatus(CS_UNKNOWN),
+	alreadySaving(false),
+	m_device_data(new DCDeviceData(this)),
+	m_libdcLog(false)
 {
 	m_instance = this;
 	m_lastDevicePixelRatio = qApp->devicePixelRatio();
 	connect(qobject_cast<QApplication *>(QApplication::instance()), &QApplication::applicationStateChanged, this, &QMLManager::applicationStateChanged);
-	appendTextToLog(getUserAgent());
+
+	QString libdcLogFileName = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + "/libdivecomputer.log";
+	logfile_name = strdup(libdcLogFileName.toUtf8().data());
+#if defined(Q_OS_ANDROID)
+	appLogFileName = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + "/subsurface.log";
+	appLogFile.setFileName(appLogFileName);
+	if (!appLogFile.open(QIODevice::ReadWrite|QIODevice::Truncate)) {
+		appLogFileOpen = false;
+		appendTextToLog("Failed to open logfile " + appLogFileName
+				+ " at " + QDateTime::currentDateTime().toString());
+	} else {
+		appLogFileOpen = true;
+		appendTextToLog("Successfully opened logfile " + appLogFileName
+				+ " at " + QDateTime::currentDateTime().toString());
+	}
+#endif
+	appendTextToLog("Starting " + getUserAgent());
 	appendTextToLog(QStringLiteral("build with Qt Version %1, runtime from Qt Version %2").arg(QT_VERSION_STR).arg(qVersion()));
-	qDebug() << "Starting" << getUserAgent();
-	qDebug() << QStringLiteral("build with Qt Version %1, runtime from Qt Version %2").arg(QT_VERSION_STR).arg(qVersion());
 	setStartPageText(tr("Starting..."));
-	setAccessingCloud(-1);
+
+#if defined(BT_SUPPORT)
+	// ensure that we start the BTDiscovery - this should be triggered by the export of the class
+	// to QML, but that doesn't seem to always work
+	BTDiscovery *btDiscovery = BTDiscovery::instance();
+	m_btEnabled = btDiscovery->btAvailable();
+	connect(&btDiscovery->localBtDevice, &QBluetoothLocalDevice::hostModeStateChanged,
+		this, &QMLManager::btHostModeChange);
+#else
+	m_btEnabled = false;
+#endif
 	setShowPin(false);
 	// create location manager service
 	locationProvider = new GpsLocation(&appendTextToLogStandalone, this);
+	progress_callback = &progressCallback;
 	connect(locationProvider, SIGNAL(haveSourceChanged()), this, SLOT(hasLocationSourceChanged()));
 	setLocationServiceAvailable(locationProvider->hasLocationsSource());
 	set_git_update_cb(&gitProgressCB);
@@ -125,7 +175,6 @@ void QMLManager::applicationStateChanged(Qt::ApplicationState state)
 	stateText.append(" and ");
 	stateText.append((unsaved_changes() ? QLatin1Literal("") : QLatin1Literal("no ")) + QLatin1Literal("unsaved changes"));
 	appendTextToLog(stateText);
-	qDebug() << QString::number(timer.elapsed() / 1000.0,'f', 3) << ":" << stateText;
 
 	if (!alreadySaving && state == Qt::ApplicationInactive && unsaved_changes()) {
 		// FIXME
@@ -139,18 +188,31 @@ void QMLManager::applicationStateChanged(Qt::ApplicationState state)
 void QMLManager::openLocalThenRemote(QString url)
 {
 	clear_dive_file_data();
+	setNotificationText(tr("Open local dive data file"));
 	QByteArray fileNamePrt = QFile::encodeName(url);
 	bool glo = prefs.git_local_only;
 	prefs.git_local_only = true;
 	int error = parse_file(fileNamePrt.data());
-	setAccessingCloud(-1);
 	prefs.git_local_only = glo;
 	if (error) {
 		appendTextToLog(QStringLiteral("loading dives from cache failed %1").arg(error));
+		setNotificationText(tr("Opening local data file failed"));
+		/* there can be 2 reasons for this:
+		 * 1) we have cloud credentials, but there is no local repo (yet).
+		 *    This implies that the PIN verify is still to be done.
+		 * 2) we are in a very clean state after installing the app, and
+		 *    want to use a NO CLOUD setup. The intial repo has no initial
+		 *    commit in it, so its master branch does not yet exist. We do not
+		 *    care about this, as the very first commit of dive data to the
+		 *    no cloud repo solves this.
+		 */
+
+		if (credentialStatus() != CS_NOCLOUD)
+			setCredentialStatus(CS_NEED_TO_VERIFY);
 	} else {
-		// if we can load from the cache, we know that we have at least a valid email
-		if (credentialStatus() == UNKNOWN)
-			setCredentialStatus(VALID_EMAIL);
+		// if we can load from the cache, we know that we have a valid cloud account
+		if (credentialStatus() == CS_UNKNOWN)
+			setCredentialStatus(CS_VERIFIED);
 		prefs.unit_system = git_prefs.unit_system;
 		if (git_prefs.unit_system == IMPERIAL)
 			git_prefs.units = IMPERIAL_units;
@@ -166,9 +228,14 @@ void QMLManager::openLocalThenRemote(QString url)
 		DiveListModel::instance()->clear();
 		DiveListModel::instance()->addAllDives();
 		appendTextToLog(QStringLiteral("%1 dives loaded from cache").arg(dive_table.nr));
+		setNotificationText(tr("%1 dives loaded from local dive data file").arg(dive_table.nr));
 	}
-	if (oldStatus() == NOCLOUD) {
-		// if we switch to credentials from NOCLOUD, we take things online temporarily
+	if (credentialStatus() == CS_NEED_TO_VERIFY) {
+		appendTextToLog(QStringLiteral("have cloud credentials, but still needs PIN"));
+		setShowPin(true);
+	}
+	if (oldStatus() == CS_NOCLOUD) {
+		// if we switch to credentials from CS_NOCLOUD, we take things online temporarily
 		prefs.git_local_only = false;
 		appendTextToLog(QStringLiteral("taking things online to be able to switch to cloud account"));
 	}
@@ -189,12 +256,49 @@ void QMLManager::mergeLocalRepo()
 	process_dives(true, false);
 }
 
+void QMLManager::clearCredentials()
+{
+	setCloudUserName(NULL);
+	setCloudPassword(NULL);
+	setCloudPin(NULL);
+}
+
+void QMLManager::cancelCredentialsPinSetup()
+{
+	/*
+	 * The user selected <cancel> on the final stage of the
+	 * cloud account generation (entering the emailed PIN).
+	 *
+	 * Resets the cloud credential status to CS_UNKNOWN, resulting
+	 * in a return to the first crededentials page, with the
+	 * email and passwd still filled in. In case of a cancel
+	 * of registration (from the PIN page), the email address
+	 * was probably misspelled, so the user never received a PIN to
+	 * complete the process.
+	 *
+	 * Notice that this function is also used to switch to a different
+	 * cloud account, so the name is not perfect.
+	 */
+	QSettings s;
+
+	setCredentialStatus(CS_UNKNOWN);
+	s.beginGroup("CloudStorage");
+	s.setValue("email", cloudUserName());
+	s.setValue("password", cloudPassword());
+	s.setValue("cloud_verification_status", credentialStatus());
+	s.sync();
+	setStartPageText(tr("Starting..."));
+
+	setShowPin(false);
+}
+
 void QMLManager::finishSetup()
 {
 	// Initialize cloud credentials.
 	setCloudUserName(prefs.cloud_storage_email);
 	setCloudPassword(prefs.cloud_storage_password);
 	setSyncToCloud(!prefs.git_local_only);
+	setCredentialStatus((cloud_status_qml) prefs.cloud_verification_status);
 	// if the cloud credentials are valid, we should get the GPS Webservice ID as well
 	QString url;
 	if (!cloudUserName().isEmpty() &&
@@ -204,22 +308,23 @@ void QMLManager::finishSetup()
 		// but we need to make sure we stay the only ones accessing git storage
 		alreadySaving = true;
 		openLocalThenRemote(url);
-	} else if (!same_string(existing_filename, "")) {
-		setCredentialStatus(NOCLOUD);
+	} else if (!same_string(existing_filename, "") && credentialStatus() != CS_UNKNOWN) {
+		setCredentialStatus(CS_NOCLOUD);
+		saveCloudCredentials();
 		appendTextToLog(tr("working in no-cloud mode"));
 		int error = parse_file(existing_filename);
 		if (error) {
 			// we got an error loading the local file
 			appendTextToLog(QString("got error %2 when parsing file %1").arg(existing_filename, get_error_string()));
+			setNotificationText(tr("Error parsing local storage, giving up"));
 			set_filename(NULL, "");
 		} else {
 			// successfully opened the local file, now add thigs to the dive list
 			consumeFinishedLoad(0);
-			setAccessingCloud(-1);
 			appendTextToLog(QString("working in no-cloud mode, finished loading %1 dives from %2").arg(dive_table.nr).arg(existing_filename));
 		}
 	} else {
-		setCredentialStatus(INCOMPLETE);
+		setCredentialStatus(CS_UNKNOWN);
 		appendTextToLog(tr("no cloud credentials"));
 		setStartPageText(RED_FONT + tr("Please enter valid cloud credentials.") + END_FONT);
 	}
@@ -229,6 +334,10 @@ void QMLManager::finishSetup()
 
 QMLManager::~QMLManager()
 {
+#if defined(Q_OS_ANDROID)
+	if (appLogFileOpen)
+		appLogFile.close();
+#endif
 	m_instance = NULL;
 }
 
@@ -251,33 +360,50 @@ void QMLManager::saveCloudCredentials()
 {
 	QSettings s;
 	bool cloudCredentialsChanged = false;
+	// make sure we only have letters, numbers, and +-_. in password and email address
+	QRegularExpression regExp("^[a-zA-Z0-9@.+_-]+$");
+	QString cloudPwd = cloudPassword();
+	QString cloudUser = cloudUserName();
+	if (credentialStatus() != CS_NOCLOUD) {
+		// in case of NO_CLOUD, the email address + passwd do not care, so do not check it.
+		if (cloudPwd.isEmpty() || !regExp.match(cloudPwd).hasMatch() || !regExp.match(cloudUser).hasMatch()) {
+			setStartPageText(RED_FONT + tr("Cloud storage email and password can only consist of letters, numbers, and '.', '-', '_', and '+'.") + END_FONT);
+			return;
+		}
+		// use the same simplistic regex as the backend to check email addresses
+		regExp = QRegularExpression("^[a-zA-Z0-9.+_-]+@[a-zA-Z0-9.+_-]+\\.[a-zA-Z0-9]+");
+		if (!regExp.match(cloudUser).hasMatch()) {
+			setStartPageText(RED_FONT + tr("Invalid format for email address") + END_FONT);
+			return;
+		}
+	}
 	s.beginGroup("CloudStorage");
-	s.setValue("email", cloudUserName());
-	s.setValue("password", cloudPassword());
+	s.setValue("email", cloudUser);
+	s.setValue("password", cloudPwd);
+	s.setValue("cloud_verification_status", credentialStatus());
 	s.sync();
-	if (!same_string(prefs.cloud_storage_email, qPrintable(cloudUserName()))) {
+	if (!same_string(prefs.cloud_storage_email, qPrintable(cloudUser))) {
 		free(prefs.cloud_storage_email);
-		prefs.cloud_storage_email = strdup(qPrintable(cloudUserName()));
+		prefs.cloud_storage_email = strdup(qPrintable(cloudUser));
 		cloudCredentialsChanged = true;
 	}
 
-	cloudCredentialsChanged |= !same_string(prefs.cloud_storage_password, qPrintable(cloudPassword()));
+	cloudCredentialsChanged |= !same_string(prefs.cloud_storage_password, qPrintable(cloudPwd));
 
-	if (!cloudCredentialsChanged) {
+	if (credentialStatus() != CS_NOCLOUD && !cloudCredentialsChanged) {
 		// just go back to the dive list
 		setCredentialStatus(oldStatus());
 	}
 
-	if (!same_string(prefs.cloud_storage_password, qPrintable(cloudPassword()))) {
+	if (!same_string(prefs.cloud_storage_password, qPrintable(cloudPwd))) {
 		free(prefs.cloud_storage_password);
-		prefs.cloud_storage_password = strdup(qPrintable(cloudPassword()));
+		prefs.cloud_storage_password = strdup(qPrintable(cloudPwd));
 	}
-	if (cloudUserName().isEmpty() || cloudPassword().isEmpty()) {
+	if (cloudUser.isEmpty() || cloudPwd.isEmpty()) {
 		setStartPageText(RED_FONT + tr("Please enter valid cloud credentials.") + END_FONT);
 	} else if (cloudCredentialsChanged) {
 		// let's make sure there are no unsaved changes
 		saveChangesLocal();
-
 		free(prefs.userid);
 		prefs.userid = NULL;
 		syncLoadFromCloud();
@@ -309,7 +435,6 @@ void QMLManager::checkCredentialsAndExecute(execute_function_type execute)
 	// and (if we haven't done so) load the dive list
 	if (!same_string(prefs.cloud_storage_email, "") &&
 	    !same_string(prefs.cloud_storage_password, "")) {
-		setAccessingCloud(0);
 		setStartPageText(tr("Testing cloud credentials"));
 		appendTextToLog("Have credentials, let's see if they are valid");
 		CloudStorageAuthenticate *csa = new CloudStorageAuthenticate(this);
@@ -330,9 +455,14 @@ void QMLManager::checkCredentialsAndExecute(execute_function_type execute)
 		}
 		myTimer.stop();
 		setCloudPin("");
-		if (prefs.cloud_verification_status != CS_VERIFIED) {
+		if (prefs.cloud_verification_status == CS_INCORRECT_USER_PASSWD) {
+			appendTextToLog(QStringLiteral("Incorrect cloud credentials"));
+			setStartPageText(RED_FONT + tr("Incorrect cloud credentials") + END_FONT);
+			revertToNoCloudIfNeeded();
+			return;
+		} else if (prefs.cloud_verification_status != CS_VERIFIED) {
 			// here we need to enter the PIN
-			appendTextToLog(QStringLiteral("Need to verify the email address - enter PIN in desktop app"));
+			appendTextToLog(QStringLiteral("Need to verify the email address - enter PIN"));
 			setStartPageText(RED_FONT + tr("Cannot connect to cloud storage - cloud account not verified") + END_FONT);
 			revertToNoCloudIfNeeded();
 			setShowPin(true);
@@ -366,7 +496,7 @@ void QMLManager::provideAuth(QNetworkReply *reply, QAuthenticator *auth)
 		// OK, credentials have been tried and didn't work, so they are invalid
 		appendTextToLog("Cloud credentials are invalid");
 		setStartPageText(RED_FONT + tr("Cloud credentials are invalid") + END_FONT);
-		setCredentialStatus(INVALID);
+		setCredentialStatus(CS_INCORRECT_USER_PASSWD);
 		reply->disconnect();
 		reply->abort();
 		reply->deleteLater();
@@ -380,21 +510,21 @@ void QMLManager::handleSslErrors(const QList<QSslError> &errors)
 {
 	setStartPageText(RED_FONT + tr("Cannot open cloud storage: Error creating https connection") + END_FONT);
 	Q_FOREACH (QSslError e, errors) {
-		qDebug() << e.errorString();
+		appendTextToLog(e.errorString());
 	}
 	reply->abort();
 	reply->deleteLater();
-	setAccessingCloud(-1);
+	setNotificationText(QStringLiteral(""));
 }
 
 void QMLManager::handleError(QNetworkReply::NetworkError nError)
 {
 	QString errorString = reply->errorString();
-	qDebug() << "handleError" << nError << errorString;
+	appendTextToLog(QStringLiteral("handleError ") + nError + QStringLiteral(": ") + errorString);
 	setStartPageText(RED_FONT + tr("Cannot open cloud storage: %1").arg(errorString) + END_FONT);
 	reply->abort();
 	reply->deleteLater();
-	setAccessingCloud(-1);
+	setNotificationText(QStringLiteral(""));
 }
 
 void QMLManager::retrieveUserid()
@@ -407,7 +537,7 @@ void QMLManager::retrieveUserid()
 		revertToNoCloudIfNeeded();
 		return;
 	}
-	setCredentialStatus(VALID);
+	setCredentialStatus(CS_VERIFIED);
 	QString userid(prefs.userid);
 	if (userid.isEmpty()) {
 		if (same_string(prefs.cloud_storage_email, "") || same_string(prefs.cloud_storage_password, "")) {
@@ -426,18 +556,12 @@ void QMLManager::retrieveUserid()
 		s.setValue("subsurface_webservice_uid", prefs.userid);
 		s.sync();
 	}
-	setCredentialStatus(VALID);
-	setStartPageText("Cloud credentials valid, loading dives...");
-	git_storage_update_progress(true, "load dives with valid credentials");
+	setCredentialStatus(CS_VERIFIED);
+	setStartPageText(tr("Cloud credentials valid, loading dives..."));
 	// this only gets called with "alreadySaving" already locked
 	loadDivesWithValidCredentials();
 }
 
-void QMLManager::loadDiveProgress(int percent)
-{
-	setAccessingCloud(percent);
-}
-
 void QMLManager::loadDivesWithValidCredentials()
 {
 	QString url;
@@ -454,7 +578,6 @@ void QMLManager::loadDivesWithValidCredentials()
 	const char *branch;
 	int error;
 	if (check_git_sha(fileNamePrt.data(), &git, &branch) == 0) {
-		qDebug() << "local cache was current, no need to modify dive list";
 		appendTextToLog("Cloud sync shows local cache was current");
 		goto successful_exit;
 	}
@@ -468,16 +591,16 @@ void QMLManager::loadDivesWithValidCredentials()
 		appendTextToLog(QString("didn't receive valid git repo, try again"));
 		error = parse_file(fileNamePrt.data());
 	}
-	setAccessingCloud(-1);
 	if (!error) {
 		report_error("filename is now %s", fileNamePrt.data());
-		const char *error_string = get_error_string();
-		appendTextToLog(error_string);
+		QString errorString(get_error_string());
+		appendTextToLog(errorString);
 		set_filename(fileNamePrt.data(), true);
 	} else {
 		report_error("failed to open file %s", fileNamePrt.data());
 		QString errorString(get_error_string());
 		appendTextToLog(errorString);
+		setNotificationText(errorString);
 		revertToNoCloudIfNeeded();
 		return;
 	}
@@ -488,8 +611,8 @@ successful_exit:
 	setLoadFromCloud(true);
 	// if we came from local storage mode, let's merge the local data into the local cache
 	// for the remote data - which then later gets merged with the remote data if necessary
-	if (oldStatus() == NOCLOUD) {
-		git_storage_update_progress(false, "import dives from nocloud local storage");
+	if (oldStatus() == CS_NOCLOUD) {
+		git_storage_update_progress(qPrintable(tr("Loading dives from local storage ('no cloud' mode)")));
 		dive_table.preexisting = dive_table.nr;
 		mergeLocalRepo();
 		DiveListModel::instance()->clear();
@@ -501,7 +624,6 @@ successful_exit:
 			prefs.git_local_only = syncToCloud();
 		}
 	}
-	setAccessingCloud(-1);
 	// if we got here just for an initial connection to the cloud, reset to offline
 	if (currentGitLocalOnly) {
 		currentGitLocalOnly = false;
@@ -517,11 +639,11 @@ void QMLManager::revertToNoCloudIfNeeded()
 		currentGitLocalOnly = false;
 		prefs.git_local_only = true;
 	}
-	if (oldStatus() == NOCLOUD) {
+	if (oldStatus() == CS_NOCLOUD) {
 		// we tried to switch to a cloud account and had previously used local data,
 		// but connecting to the cloud account (and subsequently merging the local
 		// and cloud data) failed - so let's delete the cloud credentials and go
-		// back to NOCLOUD mode in order to prevent us from losing the locally stored
+		// back to CS_NOCLOUD mode in order to prevent us from losing the locally stored
 		// dives
 		if (syncToCloud() == false) {
 			appendTextToLog(QStringLiteral("taking things back offline since sync with cloud failed"));
@@ -533,11 +655,10 @@ void QMLManager::revertToNoCloudIfNeeded()
 		prefs.cloud_storage_password = NULL;
 		setCloudUserName("");
 		setCloudPassword("");
-		setCredentialStatus(INCOMPLETE);
+		setCredentialStatus(CS_NOCLOUD);
 		set_filename(NOCLOUD_LOCALSTORAGE, true);
 		setStartPageText(RED_FONT + tr("Failed to connect to cloud server, reverting to no cloud status") + END_FONT);
 	}
-	setAccessingCloud(-1);
 	alreadySaving = false;
 }
 
@@ -574,12 +695,12 @@ void QMLManager::refreshDiveList()
 static void setupDivesite(struct dive *d, struct dive_site *ds, double lat, double lon, const char *locationtext)
 {
 	if (ds) {
-		ds->latitude.udeg = lat * 1000000;
-		ds->longitude.udeg = lon * 1000000;
+		ds->latitude.udeg = lrint(lat * 1000000);
+		ds->longitude.udeg = lrint(lon * 1000000);
 	} else {
 		degrees_t latData, lonData;
-		latData.udeg = lat;
-		lonData.udeg = lon;
+		latData.udeg = lrint(lat);
+		lonData.udeg = lrint(lon);
 		d->dive_site_uuid = create_dive_site_with_gps(locationtext, latData, lonData, d->when);
 	}
 }
@@ -606,7 +727,7 @@ bool QMLManager::checkDate(DiveObjectHelper *myDive, struct dive * d, QString da
 		newDate = QDateTime::fromString(date, format);
 		newDate.setTimeSpec(Qt::UTC);
 		if (!newDate.isValid()) {
-			qDebug() << "unable to parse date" << date << "with the given format" << format;
+			appendTextToLog("unable to parse date " + date + " with the given format " + format);
 			QRegularExpression isoDate("\\d+-\\d+-\\d+[^\\d]+\\d+:\\d+");
 			if (date.contains(isoDate)) {
 				newDate = QDateTime::fromString(date, "yyyy-M-d h:m:s");
@@ -684,7 +805,7 @@ parsed:
 			d->dc.when = d->when = newDate.toMSecsSinceEpoch() / 1000;
 			return true;
 		}
-		qDebug() << "none of our parsing attempts worked for the date string";
+		appendTextToLog("none of our parsing attempts worked for the date string");
 	}
 	return false;
 }
@@ -718,7 +839,6 @@ bool QMLManager::checkLocation(DiveObjectHelper *myDive, struct dive *d, QString
 				}
 			} else {
 				appendTextToLog("couldn't get GPS location in time");
-				qDebug() << "still don't have a position - will need to implement some sort of callback";
 			}
 		} else {
 			// just something we can't parse, so tell the user
@@ -763,7 +883,7 @@ bool QMLManager::checkDuration(DiveObjectHelper *myDive, struct dive *d, QString
 			d->dc.sample = 0;
 			d->dc.samples = 0;
 		} else {
-			qDebug() << "changing the duration on a dive that wasn't manually added - Uh-oh";
+			appendTextToLog("Cannot change the duration on a dive that wasn't manually added");
 		}
 		return true;
 	}
@@ -794,7 +914,7 @@ bool QMLManager::checkDepth(DiveObjectHelper *myDive, dive *d, QString depth)
 // update the dive and return the notes field, stripped of the HTML junk
 void QMLManager::commitChanges(QString diveId, QString date, QString location, QString gps, QString duration, QString depth,
 			       QString airtemp, QString watertemp, QString suit, QString buddy, QString diveMaster, QString weight, QString notes,
-			       QString startpressure, QString endpressure, QString gasmix, QString cylinder)
+			       QString startpressure, QString endpressure, QString gasmix, QString cylinder, int rating, int visibility)
 {
 	struct dive *d = get_dive_by_uniq_id(diveId.toInt());
 	DiveObjectHelper *myDive = new DiveObjectHelper(d);
@@ -805,7 +925,7 @@ void QMLManager::commitChanges(QString diveId, QString date, QString location, Q
 	notes = doc.toPlainText();
 
 	if (!d) {
-		qDebug() << "don't touch this... no dive";
+		appendTextToLog("cannot commit changes: no dive");
 		return;
 	}
 	bool diveChanged = false;
@@ -867,7 +987,7 @@ void QMLManager::commitChanges(QString diveId, QString date, QString location, Q
 					size = tank_info[i].ml;
 					wp = tank_info[i].bar * 1000;
 				} else {
-					size = cuft_to_l(tank_info[i].cuft) * 1000 / bar_to_atm(psi_to_bar(tank_info[i].psi));
+					size = (int) (cuft_to_l(tank_info[i].cuft) * 1000 / bar_to_atm(psi_to_bar(tank_info[i].psi)));
 					wp = psi_to_mbar(tank_info[i].psi);
 				}
 				break;
@@ -898,6 +1018,14 @@ void QMLManager::commitChanges(QString diveId, QString date, QString location, Q
 		free(d->divemaster);
 		d->divemaster = strdup(qPrintable(diveMaster));
 	}
+	if (myDive->rating() != rating) {
+		diveChanged = true;
+		d->rating = rating;
+	}
+	if (myDive->visibility() != visibility) {
+		diveChanged = true;
+		d->visibility = visibility;
+	}
 	if (myDive->notes() != notes) {
 		diveChanged = true;
 		free(d->notes);
@@ -955,11 +1083,36 @@ void QMLManager::changesNeedSaving()
 	saveChangesCloud(false);
 #endif
 }
+
+void QMLManager::openNoCloudRepo()
+/*
+ * Open the No Cloud repo. In case this repo does not (yet)
+ * exists, create one first. When done, open the repo, which
+ * is obviously empty when just created.
+ */
+{
+	char *filename = NOCLOUD_LOCALSTORAGE;
+	const char *branch;
+	struct git_repository *git;
+
+	git = is_git_repository(filename, &branch, NULL, false);
+
+	if (git == dummy_git_repository) {
+		if (git_create_local_repo(filename))
+			appendTextToLog(get_error_string());
+		set_filename(filename, true);
+		GeneralSettingsObjectWrapper s(this);
+		s.setDefaultFilename(filename);
+		s.setDefaultFileBehavior(LOCAL_DEFAULT_FILE);
+	}
+
+	openLocalThenRemote(filename);
+}
+
 void QMLManager::saveChangesLocal()
 {
 	if (unsaved_changes()) {
-		git_storage_update_progress(true, "saving dives locally"); // reset the timers
-		if (credentialStatus() == NOCLOUD) {
+		if (credentialStatus() == CS_NOCLOUD) {
 			if (same_string(existing_filename, "")) {
 				char *filename = NOCLOUD_LOCALSTORAGE;
 				if (git_create_local_repo(filename))
@@ -968,7 +1121,6 @@ void QMLManager::saveChangesLocal()
 				GeneralSettingsObjectWrapper s(this);
 				s.setDefaultFilename(filename);
 				s.setDefaultFileBehavior(LOCAL_DEFAULT_FILE);
-				qDebug() << "setting default file to" << filename;
 			}
 		} else if (!loadFromCloud()) {
 			// this seems silly, but you need a common ancestor in the repository in
@@ -984,16 +1136,16 @@ void QMLManager::saveChangesLocal()
 		bool glo = prefs.git_local_only;
 		prefs.git_local_only = true;
 		if (save_dives(existing_filename)) {
-			appendTextToLog(get_error_string());
+			QString errorString(get_error_string());
+			appendTextToLog(errorString);
+			setNotificationText(errorString);
 			set_filename(NULL, true);
-			setAccessingCloud(-1);
 			prefs.git_local_only = glo;
 			alreadySaving = false;
 			return;
 		}
 		prefs.git_local_only = glo;
 		mark_divelist_changed(false);
-		git_storage_update_progress(false, "done with local save");
 		alreadySaving = false;
 	} else {
 		appendTextToLog("local save requested with no unsaved changes");
@@ -1011,6 +1163,7 @@ void QMLManager::saveChangesCloud(bool forceRemoteSync)
 		return;
 	}
 	// first we need to store any unsaved changes to the local repo
+	gitProgressCB("Save changes to local cache");
 	saveChangesLocal();
 
 	// if the user asked not to push to the cloud we are done
@@ -1023,20 +1176,17 @@ void QMLManager::saveChangesCloud(bool forceRemoteSync)
 	}
 
 	bool glo = prefs.git_local_only;
-	git_storage_update_progress(false, "start save change to cloud");
 	prefs.git_local_only = false;
 	alreadySaving = true;
 	loadDivesWithValidCredentials();
 	alreadySaving = false;
-	git_storage_update_progress(false, "finished syncing dive list to cloud server");
-	setAccessingCloud(-1);
 	prefs.git_local_only = glo;
 }
 
 bool QMLManager::undoDelete(int id)
 {
 	if (!deletedDive || deletedDive->id != id) {
-		qDebug() << "can't find the deleted dive";
+		appendTextToLog("Trying to undo delete but can't find the deleted dive");
 		return false;
 	}
 	if (deletedTrip)
@@ -1064,7 +1214,7 @@ void QMLManager::deleteDive(int id)
 {
 	struct dive *d = get_dive_by_uniq_id(id);
 	if (!d) {
-		qDebug() << "oops, trying to delete non-existing dive";
+		appendTextToLog("trying to delete non-existing dive");
 		return;
 	}
 	// clean up (or create) the storage for the deleted dive and trip (if applicable)
@@ -1093,6 +1243,11 @@ void QMLManager::deleteDive(int id)
 	changesNeedSaving();
 }
 
+void QMLManager::cancelDownloadDC()
+{
+	import_thread_cancelled = true;
+}
+
 QString QMLManager::addDive()
 {
 	appendTextToLog("Adding new dive.");
@@ -1168,8 +1323,7 @@ void QMLManager::appendTextToLog(const QString &newText)
 {
 	if (!timer.isValid())
 		timer.start();
-	m_logText += "\n" + QString::number(timer.elapsed() / 1000.0,'f', 3) + ": " + newText;
-	emit logTextChanged();
+	qDebug() << QString::number(timer.elapsed() / 1000.0,'f', 3) + ": " + newText;
 }
 
 bool QMLManager::locationServiceEnabled() const
@@ -1181,6 +1335,7 @@ void QMLManager::setLocationServiceEnabled(bool locationServiceEnabled)
 {
 	m_locationServiceEnabled = locationServiceEnabled;
 	locationProvider->serviceEnable(m_locationServiceEnabled);
+	emit locationServiceEnabledChanged();
 }
 
 bool QMLManager::locationServiceAvailable() const
@@ -1194,7 +1349,7 @@ bool QMLManager::locationServiceAvailable() const
 
 void QMLManager::setLocationServiceAvailable(bool locationServiceAvailable)
 {
-	qDebug() << "location service is" << (locationServiceAvailable ? "available" : "not available");
+	appendTextToLog(QStringLiteral("location service is ") + (locationServiceAvailable ? QStringLiteral("available") : QStringLiteral("not available")));
 	m_locationServiceAvailable = locationServiceAvailable;
 	emit locationServiceAvailableChanged();
 }
@@ -1213,7 +1368,7 @@ void QMLManager::setVerboseEnabled(bool verboseMode)
 {
 	m_verboseEnabled = verboseMode;
 	verbose = verboseMode;
-	qDebug() << "verbose is" << verbose;
+	appendTextToLog(QStringLiteral("verbose is ") + (verbose ? QStringLiteral("on") : QStringLiteral("off")));
 	emit verboseEnabledChanged();
 }
 
@@ -1269,9 +1424,25 @@ int QMLManager::timeThreshold() const
 void QMLManager::setTimeThreshold(int time)
 {
 	m_timeThreshold = time;
+	locationProvider->setGpsTimeThreshold(m_timeThreshold * 60);
 	emit timeThresholdChanged();
 }
 
+void QMLManager::setTheme(QString theme)
+{
+	QSettings s;
+	s.beginGroup("Theme");
+	s.setValue("currentTheme", theme);
+	emit themeChanged();
+}
+
+QString QMLManager::theme() const
+{
+	QSettings s;
+	s.beginGroup("Theme");
+	return s.value("currentTheme", "Blue").toString();
+}
+
 bool QMLManager::loadFromCloud() const
 {
 	return m_loadFromCloud;
@@ -1304,27 +1475,31 @@ void QMLManager::setStartPageText(const QString& text)
 	emit startPageTextChanged();
 }
 
-QMLManager::credentialStatus_t QMLManager::credentialStatus() const
+QMLManager::cloud_status_qml QMLManager::credentialStatus() const
 {
 	return m_credentialStatus;
 }
 
-void QMLManager::setCredentialStatus(const credentialStatus_t value)
+void QMLManager::setCredentialStatus(const cloud_status_qml value)
 {
 	if (m_credentialStatus != value) {
-		if (value == NOCLOUD)
+		setOldStatus(m_credentialStatus);
+		if (value == CS_NOCLOUD) {
 			appendTextToLog("Switching to no cloud mode");
+			set_filename(NOCLOUD_LOCALSTORAGE, true);
+			clearCredentials();
+		}
 		m_credentialStatus = value;
 		emit credentialStatusChanged();
 	}
 }
 
-QMLManager::credentialStatus_t QMLManager::oldStatus() const
+QMLManager::cloud_status_qml QMLManager::oldStatus() const
 {
 	return m_oldStatus;
 }
 
-void QMLManager::setOldStatus(const credentialStatus_t value)
+void QMLManager::setOldStatus(const cloud_status_qml value)
 {
 	if (m_oldStatus != value) {
 		m_oldStatus = value;
@@ -1361,15 +1536,15 @@ QString QMLManager::getVersion() const
 	return versionRe.cap(1);
 }
 
-int QMLManager::accessingCloud() const
+QString QMLManager::notificationText() const
 {
-	return m_accessingCloud;
+	return m_notificationText;
 }
 
-void QMLManager::setAccessingCloud(int status)
+void QMLManager::setNotificationText(QString text)
 {
-	m_accessingCloud = status;
-	emit accessingCloudChanged();
+	m_notificationText = text;
+	emit notificationTextChanged();
 }
 
 bool QMLManager::syncToCloud() const
@@ -1512,3 +1687,97 @@ void QMLManager::setShowPin(bool enable)
 	m_showPin = enable;
 	emit showPinChanged();
 }
+
+QString QMLManager::progressMessage() const
+{
+	return m_progressMessage;
+}
+
+void QMLManager::setProgressMessage(QString text)
+{
+	m_progressMessage = text;
+	emit progressMessageChanged();
+}
+
+bool QMLManager::libdcLog() const
+{
+	return m_libdcLog;
+}
+
+void QMLManager::setLibdcLog(bool value)
+{
+	m_libdcLog = value;
+	DCDeviceData::instance()->setSaveLog(value);
+	emit libdcLogChanged();
+}
+
+bool QMLManager::developer() const
+{
+	return m_developer;
+}
+
+void QMLManager::setDeveloper(bool value)
+{
+	m_developer = value;
+	emit developerChanged();
+}
+
+bool QMLManager::btEnabled() const
+{
+	return m_btEnabled;
+}
+
+void QMLManager::setBtEnabled(bool value)
+{
+	m_btEnabled = value;
+}
+
+#if defined (Q_OS_ANDROID)
+
+void writeToAppLogFile(QString logText)
+{
+	// write to storage and flush so that the data doesn't get lost
+	logText.append("\n");
+	QMLManager *self = QMLManager::instance();
+	if (self) {
+		self->writeToAppLogFile(logText);
+	}
+}
+
+void QMLManager::writeToAppLogFile(QString logText)
+{
+	if (appLogFileOpen) {
+		appLogFile.write(logText.toUtf8().data());
+		appLogFile.flush();
+	}
+}
+
+//HACK to color the system bar on Android, use qtandroidextras and call the appropriate Java methods
+//this code is based on code in the Kirigami example app for Android (under LGPL-2) Copyright 2017 Marco Martin
+
+#include <QtAndroid>
+
+// there doesn't appear to be an include that defines these in an easily accessible way
+// WindowManager.LayoutParams
+#define FLAG_TRANSLUCENT_STATUS 0x04000000
+#define FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS 0x80000000
+// View
+#define SYSTEM_UI_FLAG_LIGHT_STATUS_BAR 0x00002000
+
+void QMLManager::setStatusbarColor(QColor color)
+{
+	QtAndroid::runOnAndroidThread([=]() {
+		QAndroidJniObject window = QtAndroid::androidActivity().callObjectMethod("getWindow", "()Landroid/view/Window;");
+		window.callMethod<void>("addFlags", "(I)V", FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
+		window.callMethod<void>("clearFlags", "(I)V", FLAG_TRANSLUCENT_STATUS);
+		window.callMethod<void>("setStatusBarColor", "(I)V", color.rgba());
+		window.callMethod<void>("setNavigationBarColor", "(I)V", color.rgba());
+	});
+}
+#else
+void QMLManager::setStatusbarColor(QColor color)
+{
+	// noop
+}
+
+#endif
