@@ -38,6 +38,8 @@ BUILD_NR=0
 SUBSURFACE_DESKTOP=OFF
 # Which arch should we build for?
 ARCH=arm
+# Which SDK buildtools revision is used?
+ANDROID_BUILDTOOLS_REVISION=25.0.3
 
 while [ "$#" -gt 0 ] ; do
 	case "$1" in
@@ -70,10 +72,14 @@ done
 export ARCH
 
 # Configure where we can find things here
-export ANDROID_NDK_ROOT=${ANDROID_NDK_ROOT-$SUBSURFACE_SOURCE/../android-ndk-r13b}
+export ANDROID_NDK_ROOT=${ANDROID_NDK_ROOT-$SUBSURFACE_SOURCE/../android-ndk-r14b}
 
 if [ -n "${QT5_ANDROID+X}" ] ; then
 	echo "Using Qt5 in $QT5_ANDROID"
+elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.9.1" ] ; then
+	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.9.1
+elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.9" ] ; then
+	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.9
 elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.8" ] ; then
 	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.8
 elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.7" ] ; then
@@ -83,7 +89,7 @@ elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.6" ] ; then
 elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.5" ] ; then
 	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.5
 else
-	echo "Cannot find Qt 5.7, 5.6 or 5.5 under $SUBSURFACE_SOURCE/../Qt"
+	echo "Cannot find Qt 5.5 or newer under $SUBSURFACE_SOURCE/../Qt"
 	exit 1
 fi
 
@@ -96,13 +102,13 @@ else
 fi
 
 # Which versions are we building against?
-SQLITE_VERSION=3130000
+SQLITE_VERSION=3190200
 LIBXML2_VERSION=2.9.4
 LIBXSLT_VERSION=1.1.29
 LIBZIP_VERSION=1.1.3
-LIBGIT2_VERSION=0.24.1
+LIBGIT2_VERSION=0.26.0
 LIBUSB_VERSION=1.0.20
-OPENSSL_VERSION=1.0.2h
+OPENSSL_VERSION=1.0.2l
 LIBFTDI_VERSION=1.3
 
 if [ "$ARCH" = "arm" ] ; then
@@ -114,7 +120,18 @@ elif [ "$ARCH" = "x86" ] ; then
 	BUILDCHAIN=i686-linux-android
 	OPENSSL_MACHINE=i686
 fi
-export QT5_ANDROID_BIN=${QT5_ANDROID}/android_${QT_ARCH}/bin
+
+# Verify Qt install and adjust for single-arch Qt install layout
+# (e.g. when building Qt from scratch)
+export QT5_ANDROID_CMAKE
+if [ -d "${QT5_ANDROID}/android_${QT_ARCH}/lib/cmake" ] ; then
+	export QT5_ANDROID_CMAKE=$QT5_ANDROID/android_${QT_ARCH}/lib/cmake
+elif [ -d "${QT5_ANDROID}/lib/cmake" ] ; then
+	export QT5_ANDROID_CMAKE=$QT5_ANDROID/lib/cmake
+else
+	echo "Cannot find Qt cmake configuration"
+	exit 1
+fi
 
 if [ ! -e ndk-"$ARCH" ] ; then
 	"$ANDROID_NDK_ROOT/build/tools/make_standalone_toolchain.py" --arch="$ARCH" --install-dir=ndk-"$ARCH" --api=16
@@ -140,7 +157,7 @@ else
 fi
 
 if [ ! -e sqlite-autoconf-${SQLITE_VERSION}.tar.gz ] ; then
-	wget http://www.sqlite.org/2016/sqlite-autoconf-${SQLITE_VERSION}.tar.gz
+	wget http://www.sqlite.org/2017/sqlite-autoconf-${SQLITE_VERSION}.tar.gz
 fi
 if [ ! -e sqlite-autoconf-${SQLITE_VERSION} ] ; then
 	tar -zxf sqlite-autoconf-${SQLITE_VERSION}.tar.gz
@@ -239,6 +256,9 @@ fi
 if [ ! -e "$PKG_CONFIG_LIBDIR/libgit2.pc" ] ; then
 	# We don't want to find the HTTP_Parser package of the build host by mistake
 	perl -pi -e 's/FIND_PACKAGE\(HTTP_Parser\)/#FIND_PACKAGE(HTTP_Parser)/' libgit2-${LIBGIT2_VERSION}/CMakeLists.txt
+	# also, with libgit-0.25.1 and ndk-14b we see an odd build failure with a missing rand() symbol
+	# trivial workaround is to just call lrand48()
+	perl -pi -e 's/seed = rand/seed = (int)lrand48/' libgit2-${LIBGIT2_VERSION}/src/cache.c
 	mkdir -p libgit2-build-"$ARCH"
 	pushd libgit2-build-"$ARCH"
 	cmake -DCMAKE_SYSTEM_NAME=Android -DSHA1_TYPE=builtin \
@@ -340,24 +360,22 @@ if [ ! -z "$SUBSURFACE_MOBILE" ] ; then
 	mkdir -p subsurface-mobile-build-"$ARCH"
 	cd subsurface-mobile-build-"$ARCH"
 	MOBILE_CMAKE=-DSUBSURFACE_TARGET_EXECUTABLE=MobileExecutable
+	BUILD_NAME=Subsurface-mobile
 else
 	MOBILE_CMAKE=""
 	mkdir -p subsurface-build-"$ARCH"
 	cd subsurface-build-"$ARCH"
+	BUILD_NAME=Subsurface
 fi
 
-# something in the qt-android-cmake-thingies mangles your path, so thats why we need to hard-code ant and pkg-config here.
 if [ "$PLATFORM" = "Darwin" ] ; then
-	ANT=/usr/local/bin/ant
 	FTDI=OFF
 else
-	ANT=/usr/bin/ant
 	FTDI=ON
 fi
 
 PKGCONF=$(which pkg-config)
 cmake $MOBILE_CMAKE \
-	-DQT_ANDROID_ANT=${ANT} \
 	-DPKG_CONFIG_EXECUTABLE="$PKGCONF" \
 	-DQT_ANDROID_SDK_ROOT="$ANDROID_SDK_ROOT" \
 	-DQT_ANDROID_NDK_ROOT="$ANDROID_NDK_ROOT" \
@@ -366,16 +384,16 @@ cmake $MOBILE_CMAKE \
 	-DFORCE_LIBSSH=OFF \
 	-DLIBDC_FROM_PKGCONFIG=ON \
 	-DLIBGIT2_FROM_PKGCONFIG=ON \
-	-DNO_MARBLE=ON \
 	-DNO_PRINTING=ON \
 	-DNO_USERMANUAL=ON \
 	-DFBSUPPORT=OFF \
-	-DCMAKE_PREFIX_PATH:UNINITIALIZED="$QT5_ANDROID/android_$QT_ARCH/lib/cmake" \
+	-DCMAKE_PREFIX_PATH:UNINITIALIZED="$QT5_ANDROID_CMAKE" \
 	-DCMAKE_BUILD_TYPE="$BUILD_TYPE" \
 	-DMAKE_TESTS=OFF \
 	-DFTDISUPPORT=${FTDI} \
 	-DANDROID_NATIVE_LIBSSL="$BUILDROOT/ndk-$ARCH/sysroot/usr/lib/libssl.so" \
 	-DANDROID_NATIVE_LIBCRYPT="$BUILDROOT/ndk-$ARCH/sysroot/usr/lib/libcrypto.so" \
+	-DBUILDTOOLS_REVISION="$ANDROID_BUILDTOOLS_REVISION" \
 	"$SUBSURFACE_SOURCE"
 
 # set up the version number
@@ -407,7 +425,4 @@ cp -a translations/*.qm assets/translations
 # now build Subsurface and use the rest of the command line arguments
 make "$@"
 
-#make install INSTALL_ROOT=android_build
-# bug in androiddeployqt? why is it looking for something with the builddir in it?
-#ln -fs android-libsubsurface.so-deployment-settings.json android-libsubsurface-build-${ARCH}.so-deployment-settings.json
-#$QT5_ANDROID_BIN/androiddeployqt --output android_build
+echo "Done building $BUILD_NAME for Android"
