@@ -3,12 +3,10 @@
 # build Subsurface for Win32
 #
 # this file assumes that you have installed MXE on your system
-# and installed a number of dependencies as well. Latest MXE
-# version from git may not always work for Qt5 and Subsurface. 
-# Try to select an older release version like build-2016-10-12.
+# and installed a number of dependencies as well.
 #
 # cd ~/src
-# git clone --branch build-2016-10-12 https://github.com/mxe/mxe
+# git clone https://github.com/mxe/mxe
 # cd mxe
 #
 # now create a file settings.mk
@@ -47,8 +45,8 @@
 #      /libcurl                <- from git - 7.42.1 seems to work - rename folder!
 #      /subsurface             <- current subsurface git
 #      /libdivecomputer        <- appropriate libdc/Subsurface-branch branch
-#      /marble-source          <- appropriate marble/Subsurface-branch branch
 #      /libgit2                <- libgit2 0.23.1 or similar
+#      /googlemaps             <- Google Maps plugin for QtLocation from git
 #
 # ~/src/win32                  <- build directory
 #
@@ -95,10 +93,11 @@ JOBS="-j4"
 EXECDIR=`pwd`
 BASEDIR=$(cd "$EXECDIR/.."; pwd)
 BUILDDIR=$(cd "$EXECDIR"; pwd)
+MXEDIR=${MXEDIR:-mxe}
 
 echo $BUILDDIR
 
-if [[ ! -d "$BASEDIR"/mxe ]] ; then
+if [[ ! -d "$BASEDIR"/"$MXEDIR" ]] ; then
 	echo "Please start this from the right directory "
 	echo "usually a winbuild directory parallel to the mxe directory"
 	exit 1
@@ -106,7 +105,7 @@ fi
 
 echo "Building in $BUILDDIR ..."
 
-export PATH="$BASEDIR"/mxe/usr/bin:$PATH:"$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/qt5/bin/
+export PATH="$BASEDIR"/"$MXEDIR"/usr/bin:$PATH:"$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/
 export CXXFLAGS=-std=c++11
 
 if [[ "$1" == "debug" ]] ; then
@@ -133,7 +132,7 @@ if [[ ! -d grantlee || -f build.grantlee ]] ; then
 	rm -f build.grantlee
 	mkdir -p grantlee
 	cd grantlee
-	cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
+	i686-w64-mingw32.shared-cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
 		-DCMAKE_BUILD_TYPE=$RELEASE \
 		-DBUILD_TESTS=OFF \
 		"$BASEDIR"/grantlee
@@ -151,7 +150,7 @@ if [[ ! -d libssh2 || -f build.libssh2 ]] ; then
 	mkdir -p libssh2
 	cd libssh2
 
-	cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
+	i686-w64-mingw32.shared-cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
 		-DCMAKE_BUILD_TYPE=$RELEASE \
 		-DBUILD_EXAMPLES=OFF \
 		-DBUILD_TESTING=OFF \
@@ -160,7 +159,7 @@ if [[ ! -d libssh2 || -f build.libssh2 ]] ; then
 	make $JOBS
 	make install
 	# don't install your dlls in bin, please
-	cp "$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/bin/libssh2.dll "$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/lib
+	cp "$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/bin/libssh2.dll "$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/lib
 fi
 
 
@@ -172,7 +171,7 @@ if [[ ! -d libcurl || -f build.libcurl ]] ; then
 	mkdir -p libcurl
 	cd libcurl
 	../../libcurl/configure --host=i686-w64-mingw32.shared \
-		--prefix="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/ \
+		--prefix="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/ \
 		--disable-ftp \
 		--disable-ldap \
 		--disable-ldaps \
@@ -187,7 +186,7 @@ if [[ ! -d libcurl || -f build.libcurl ]] ; then
 		--disable-smtp \
 		--disable-gopher \
 		--disable-manual \
-		--with-libssh2="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/
+		--with-libssh2="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/
 
 	# now remove building the executable
 	sed -i 's/SUBDIRS = lib src include/SUBDIRS = lib include/' Makefile
@@ -204,10 +203,10 @@ if [[ ! -d libgit2 || -f build.libgit2 ]] ; then
 	rm -f build.libgit2
 	mkdir -p libgit2
 	cd libgit2
-	cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
+	i686-w64-mingw32.shared-cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
 		-DBUILD_CLAR=OFF -DTHREADSAFE=ON \
 		-DCMAKE_BUILD_TYPE=$RELEASE \
-		-DDLLTOOL="$BASEDIR"/mxe/usr/bin/i686-w64-mingw32.shared-dlltool \
+		-DDLLTOOL="$BASEDIR"/"$MXEDIR"/usr/bin/i686-w64-mingw32.shared-dlltool \
 		"$BASEDIR"/libgit2
 	make $JOBS
 	make install
@@ -231,7 +230,7 @@ if [[ ! -d libdivecomputer || -f build.libdivecomputer ]] ; then
 
 	"$BASEDIR"/libdivecomputer/configure --host=i686-w64-mingw32.shared \
 		--enable-shared \
-		--prefix="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared
+		--prefix="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared
 	make $JOBS
 	make install
 else
@@ -244,27 +243,17 @@ else
 	echo ""
 fi
 
-# marble:
-
 cd "$BUILDDIR"
-if [[ ! -d marble || -f build.marble ]] ; then
-	rm -f build.marble
-	mkdir -p marble
-	cd marble
-	cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
-		-DCMAKE_PREFIX_PATH="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/qt5 \
-		-DQTONLY=ON -DQT5BUILD=ON \
-		-DBUILD_MARBLE_APPS=OFF -DBUILD_MARBLE_EXAMPLES=OFF \
-		-DBUILD_MARBLE_TESTS=OFF -DBUILD_MARBLE_TOOLS=OFF \
-		-DBUILD_TESTING=OFF -DWITH_DESIGNER_PLUGIN=OFF \
-		-DBUILD_WITH_DBUS=OFF \
-		-DCMAKE_BUILD_TYPE=$RELEASE \
-		"$BASEDIR"/marble-source
+if [[ ! -d googlemaps || -f build.googlemaps ]] ; then
+	rm -f build.googlemaps
+	cd "$BASEDIR"/googlemaps
+	git pull
+	cd "$BUILDDIR"
+	mkdir -p googlemaps
+	cd googlemaps
+	"$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/qmake PREFIX=$"$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared "$BASEDIR"/googlemaps/googlemaps.pro
 	make $JOBS
 	make install
-	# what the heck is marble doing?
-	mv "$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/libssrfmarblewidget"$DLL_SUFFIX".dll "$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/lib
-	mv "$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/libastro"$DLL_SUFFIX".dll "$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/lib
 fi
 
 ###############
@@ -278,18 +267,11 @@ echo "Starting Subsurface Build"
 rm -rf subsurface
 
 # first copy the Qt plugins in place
-QT_PLUGIN_DIRECTORIES="$BASEDIR/mxe/usr/i686-w64-mingw32.shared/qt5/plugins/iconengines \
-$BASEDIR/mxe/usr/i686-w64-mingw32.shared/qt5/plugins/imageformats \
-$BASEDIR/mxe/usr/i686-w64-mingw32.shared/qt5/plugins/platforms \
-$BASEDIR/mxe/usr/i686-w64-mingw32.shared/qt5/plugins/printsupport"
-
-# for some reason we aren't installing libssrfmarblewidget.dll and # Qt5Xml.dll
-# I need to figure out why and fix that, but for now just manually copy that as well
-EXTRA_MANUAL_DEPENDENCIES="$BASEDIR/mxe/usr/i686-w64-mingw32.shared/lib/libssrfmarblewidget$DLL_SUFFIX.dll \
-$BASEDIR/mxe/usr/i686-w64-mingw32.shared/qt5/bin/Qt5Xml$DLL_SUFFIX.dll"
-
-
-
+QT_PLUGIN_DIRECTORIES="$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/iconengines \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/imageformats \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/platforms \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/geoservices \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/plugins/printsupport"
 
 STAGING_DIR=$BUILDDIR/subsurface/staging
 STAGING_TESTS_DIR=$BUILDDIR/subsurface/staging_tests
@@ -303,6 +285,9 @@ do
 	mkdir -p $STAGING_TESTS_DIR/$(basename $d)
 	for f in $d/*
 	do
+		if [[ "$d" =~  geoservice ]] && [[ ! "$f" =~ googlemaps ]] ; then
+			continue
+		fi
 		if [[ "$RELEASE" == "Release" ]] && ([[ ! -f ${f//d.dll/.dll} || "$f" == "${f//d.dll/.dll}" ]]) ; then
 			cp $f $STAGING_DIR/plugins/$(basename $d)
 			cp $f $STAGING_TESTS_DIR/$(basename $d)
@@ -313,6 +298,23 @@ do
 	done
 done
 
+# next we need the QML modules
+QT_QML_MODULES="$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/qml/QtQuick.2 \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/qml/QtLocation \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/qml/QtPositioning"
+
+mkdir -p $STAGING_DIR/qml
+
+for d in $QT_QML_MODULES
+do
+	cp -a $d $STAGING_DIR/qml
+done
+
+# for some reason we aren't installing Qt5Xml.dll and Qt5Location.dll
+# I need to figure out why and fix that, but for now just manually copy that as well
+EXTRA_MANUAL_DEPENDENCIES="$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/Qt5Xml$DLL_SUFFIX.dll \
+$BASEDIR/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/bin/Qt5Location$DLL_SUFFIX.dll"
+
 for f in $EXTRA_MANUAL_DEPENDENCIES
 do
     cp $f $STAGING_DIR
@@ -321,15 +323,13 @@ done
 
 cd "$BUILDDIR"/subsurface
 
-cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
-	-DCMAKE_PREFIX_PATH="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/qt5 \
+i686-w64-mingw32.shared-cmake -DCMAKE_TOOLCHAIN_FILE="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/share/cmake/mxe-conf.cmake \
+	-DCMAKE_PREFIX_PATH="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5 \
 	-DCMAKE_BUILD_TYPE=$RELEASE \
-	-DQT_TRANSLATION_DIR="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/qt5/translations \
+	-DQT_TRANSLATION_DIR="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/qt5/translations \
 	-DMAKENSIS=i686-w64-mingw32.shared-makensis \
-	-DLIBDIVECOMPUTER_INCLUDE_DIR="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/include \
-	-DLIBDIVECOMPUTER_LIBRARIES="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/lib/libdivecomputer.dll.a \
-	-DMARBLE_INCLUDE_DIR="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/include \
-	-DMARBLE_LIBRARIES="$BASEDIR"/mxe/usr/i686-w64-mingw32.shared/lib/libssrfmarblewidget"$DLL_SUFFIX".dll \
+	-DLIBDIVECOMPUTER_INCLUDE_DIR="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/include \
+	-DLIBDIVECOMPUTER_LIBRARIES="$BASEDIR"/"$MXEDIR"/usr/i686-w64-mingw32.shared/lib/libdivecomputer.dll.a \
 	-DMAKE_TESTS=OFF \
 	"$BASEDIR"/subsurface
 
