@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "diveplannermodel.h"
 #include "core/dive.h"
 #include "core/helpers.h"
@@ -9,6 +10,8 @@
 #include <QApplication>
 #include <QTextDocument>
 
+#define UNIT_FACTOR ((prefs.units.length == units::METERS) ? 1000.0 / 60.0 : feet_to_mm(1.0) / 60.0)
+
 /* TODO: Port this to CleanerTableModel to remove a bit of boilerplate and
  * use the signal warningMessage() to communicate errors to the MainWindow.
  */
@@ -57,7 +60,7 @@ void DivePlannerPointsModel::setupStartTime()
 	startTime = QDateTime::currentDateTimeUtc().addSecs(3600 + gettimezoneoffset());
 	if (dive_table.nr) {
 		struct dive *d = get_dive(dive_table.nr - 1);
-		time_t ends = d->when + d->duration.seconds;
+		time_t ends = dive_endtime(d);
 		time_t diff = ends - startTime.toTime_t();
 		if (diff > 0) {
 			startTime = startTime.addSecs(diff + 3600);
@@ -178,7 +181,14 @@ QStringList &DivePlannerPointsModel::getGasList()
 		cylinder_t *cyl = &displayed_dive.cylinder[i];
 		if (cylinder_nodata(cyl))
 			break;
-		list.push_back(get_gas_string(cyl->gasmix));
+		/* Check if we have the same gasmix two or more times
+		 * If yes return more verbose string */
+		int same_gas = same_gasmix_cylinder(cyl, i, &displayed_dive, true);
+		if (same_gas == -1)
+			list.push_back(get_gas_string(cyl->gasmix));
+		else
+			list.push_back(get_gas_string(cyl->gasmix) + QString(" (%1 %2 ").arg(tr("cyl.")).arg(i + 1) +
+				cyl->type.description + ")");
 	}
 	return list;
 }
@@ -207,7 +217,7 @@ void DivePlannerPointsModel::setPlanMode(Mode m)
 	// the planner may reset our GF settings that are used to show deco
 	// reset them to what's in the preferences
 	if (m != PLAN) {
-		set_gf(prefs.gflow, prefs.gfhigh, prefs.gf_low_at_maxdepth);
+		set_gf(prefs.gflow, prefs.gfhigh);
 		set_vpmb_conservatism(prefs.vpmb_conservatism);
 	}
 }
@@ -255,7 +265,15 @@ QVariant DivePlannerPointsModel::data(const QModelIndex &index, int role) const
 			else
 				return p.time / 60;
 		case GAS:
-			return get_gas_string(displayed_dive.cylinder[p.cylinderid].gasmix);
+			/* Check if we have the same gasmix two or more times
+			 * If yes return more verbose string */
+			int same_gas = same_gasmix_cylinder(&displayed_dive.cylinder[p.cylinderid], p.cylinderid, &displayed_dive, true);
+			if (same_gas == -1)
+				return get_gas_string(displayed_dive.cylinder[p.cylinderid].gasmix);
+			else
+				return get_gas_string(displayed_dive.cylinder[p.cylinderid].gasmix) +
+					QString(" (%1 %2 ").arg(tr("cyl.")).arg(p.cylinderid + 1) +
+						displayed_dive.cylinder[p.cylinderid].type.description + ")";
 		}
 	} else if (role == Qt::DecorationRole) {
 		switch (index.column()) {
@@ -319,6 +337,9 @@ bool DivePlannerPointsModel::setData(const QModelIndex &index, const QVariant &v
 		case GAS:
 			if (value.toInt() >= 0 && value.toInt() < MAX_CYLINDERS)
 				p.cylinderid = value.toInt();
+				/* Did we change the start (dp 0) cylinder to another cylinderid than 0? */
+				if (value.toInt() != 0 && index.row() == 0)
+					CylindersModel::instance()->moveAtFirst(value.toInt());
 			CylindersModel::instance()->updateTrashIcon();
 			break;
 		}
@@ -327,7 +348,7 @@ bool DivePlannerPointsModel::setData(const QModelIndex &index, const QVariant &v
 	return QAbstractItemModel::setData(index, value, role);
 }
 
-void DivePlannerPointsModel::gaschange(const QModelIndex &index, int newcylinderid)
+void DivePlannerPointsModel::gasChange(const QModelIndex &index, int newcylinderid)
 {
 	int i = index.row(), oldcylinderid = divepoints[i].cylinderid;
 	while (i < rowCount() && oldcylinderid == divepoints[i].cylinderid)
@@ -335,6 +356,13 @@ void DivePlannerPointsModel::gaschange(const QModelIndex &index, int newcylinder
 	emitDataChanged();
 }
 
+void DivePlannerPointsModel::cylinderRenumber(int mapping[])
+{
+	for (int i = 0; i < rowCount(); i++)
+		divepoints[i].cylinderid = mapping[divepoints[i].cylinderid];
+	emitDataChanged();
+}
+
 QVariant DivePlannerPointsModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
 	if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
@@ -493,6 +521,41 @@ void DivePlannerPointsModel::setLastStop6m(bool value)
 	emitDataChanged();
 }
 
+void DivePlannerPointsModel::setAscrate75(int rate)
+{
+	auto planner = SettingsObjectWrapper::instance()->planner_settings;
+	planner->setAscrate75(lrint(rate * UNIT_FACTOR));
+	emitDataChanged();
+}
+
+void DivePlannerPointsModel::setAscrate50(int rate)
+{
+	auto planner = SettingsObjectWrapper::instance()->planner_settings;
+	planner->setAscrate50(lrint(rate * UNIT_FACTOR));
+	emitDataChanged();
+}
+
+void DivePlannerPointsModel::setAscratestops(int rate)
+{
+	auto planner = SettingsObjectWrapper::instance()->planner_settings;
+	planner->setAscratestops(lrint(rate * UNIT_FACTOR));
+	emitDataChanged();
+}
+
+void DivePlannerPointsModel::setAscratelast6m(int rate)
+{
+	auto planner = SettingsObjectWrapper::instance()->planner_settings;
+	planner->setAscratelast6m(lrint(rate * UNIT_FACTOR));
+	emitDataChanged();
+}
+
+void DivePlannerPointsModel::setDescrate(int rate)
+{
+	auto planner = SettingsObjectWrapper::instance()->planner_settings;
+	planner->setDescrate(lrint(rate * UNIT_FACTOR));
+	emitDataChanged();
+}
+
 void DivePlannerPointsModel::setVerbatim(bool value)
 {
 	auto planner = SettingsObjectWrapper::instance()->planner_settings;
@@ -521,6 +584,13 @@ void DivePlannerPointsModel::setDisplayTransitions(bool value)
 	emitDataChanged();
 }
 
+void DivePlannerPointsModel::setDisplayVariations(bool value)
+{
+	auto planner = SettingsObjectWrapper::instance()->planner_settings;
+	planner->setDisplayVariations(value);
+	emitDataChanged();
+}
+
 void DivePlannerPointsModel::setDecoMode(int mode)
 {
 	auto planner = SettingsObjectWrapper::instance()->planner_settings;
@@ -686,12 +756,15 @@ void DivePlannerPointsModel::editStop(int row, divedatapoint newData)
 	 * When moving divepoints rigorously, we might end up with index
 	 * out of range, thus returning the last one instead.
 	 */
+	int old_first_cylid = divepoints[0].cylinderid;
 	if (row >= divepoints.count())
 		return;
 	divepoints[row] = newData;
 	std::sort(divepoints.begin(), divepoints.end(), divePointsLessThan);
 	if (updateMaxDepth())
 		CylindersModel::instance()->updateBestMixes();
+	if (divepoints[0].cylinderid != old_first_cylid)
+		CylindersModel::instance()->moveAtFirst(divepoints[0].cylinderid);
 	emitDataChanged();
 }
 
@@ -720,6 +793,8 @@ void DivePlannerPointsModel::remove(const QModelIndex &index)
 	if (!dp.entered)
 		return;
 
+	int old_first_cylid = divepoints[0].cylinderid;
+
 /* TODO: this seems so wrong.
  * We can't do this here if we plan to use QML on mobile
  * as mobile has no ControlModifier.
@@ -739,6 +814,8 @@ void DivePlannerPointsModel::remove(const QModelIndex &index)
 	}
 	endRemoveRows();
 	CylindersModel::instance()->updateTrashIcon();
+	if (divepoints[0].cylinderid != old_first_cylid)
+		CylindersModel::instance()->moveAtFirst(divepoints[0].cylinderid);
 }
 
 struct diveplan &DivePlannerPointsModel::getDiveplan()
@@ -822,7 +899,7 @@ void DivePlannerPointsModel::createTemporaryPlan()
 	}
 
 	// what does the cache do???
-	char *cache = NULL;
+	struct deco_state *cache = NULL;
 	struct divedatapoint *dp = NULL;
 	for (int i = 0; i < MAX_CYLINDERS; i++) {
 		cylinder_t *cyl = &displayed_dive.cylinder[i];
@@ -841,7 +918,9 @@ void DivePlannerPointsModel::createTemporaryPlan()
 	dump_plan(&diveplan);
 #endif
 	if (recalcQ() && !diveplan_empty(&diveplan)) {
-		plan(&diveplan, &cache, isPlanner(), false);
+		struct decostop stoptable[60];
+		plan(&diveplan, &displayed_dive, DECOTIMESTEP, stoptable, &cache, isPlanner(), false);
+		computeVariations();
 		emit calculatedPlanNotes();
 	}
 	// throw away the cache
@@ -867,17 +946,132 @@ void DivePlannerPointsModel::saveDuplicatePlan()
 	createPlan(true);
 }
 
+struct divedatapoint * DivePlannerPointsModel::cloneDiveplan(struct diveplan *plan_copy)
+{
+	divedatapoint *src, *last_segment;
+	divedatapoint **dp;
+
+	src = diveplan.dp;
+	*plan_copy = diveplan;
+	dp = &plan_copy->dp;
+	while (src && (!src->time || src->entered)) {
+		*dp = (struct divedatapoint *)malloc(sizeof(struct divedatapoint));
+		**dp = *src;
+		dp = &(*dp)->next;
+		src = src->next;
+	}
+	(*dp) = NULL;
+
+	last_segment = plan_copy->dp;
+	while (last_segment->next->next)
+		last_segment = last_segment->next;
+	return last_segment;
+}
+
+int DivePlannerPointsModel::analyzeVariations(struct decostop *min, struct decostop *mid, struct decostop *max, const char *unit)
+{
+	int leftsum = 0;
+	int rightsum = 0;
+	while (mid->depth > min->depth)
+		++mid;
+	while (max->depth > mid->depth)
+		++max;
+
+	while (mid->depth) {
+		int left = mid->time - min->time;
+		leftsum += left;
+		int right = max->time - mid->time;
+		rightsum += right;
+#ifdef SHOWSTOPVARIATIONS
+		if (min->time + mid->time + max->time)
+			printf("%dm: %dmin + %ds/%s +- %ds/%s\n", mid->depth / 1000,
+			       (mid->time + 1)/60,
+			       (left + right) / 2, unit,
+			       (right - left) / 2, unit);
+#else
+		(void) unit;
+#endif
+		++min;
+		++mid;
+		++max;
+	}
+#ifdef SHOWSTOPVARIATIONS
+	printf("Total + %d:%02d/%s +- %d s/%s\n\n", FRACTION((leftsum + rightsum) / 2, 60), unit,
+						       (rightsum - leftsum) / 2, unit);
+#endif
+	return (leftsum + rightsum) / 2;
+}
+
+void DivePlannerPointsModel::computeVariations()
+{
+	bool oldRecalc = setRecalc(false);
+	struct dive *dive = alloc_dive();
+	copy_dive(&displayed_dive, dive);
+	struct decostop original[60], deeper[60], shallower[60], shorter[60], longer[60];
+	struct deco_state *cache = NULL, *save = NULL;
+	struct diveplan plan_copy;
+	struct divedatapoint *last_segment;
+
+	if(in_planner() && prefs.display_variations) {
+		cache_deco_state(&save);
+		cloneDiveplan(&plan_copy);
+		plan(&plan_copy, dive, 1, original, &cache, true, false);
+		free_dps(&plan_copy);
+		restore_deco_state(save, false);
+
+		last_segment = cloneDiveplan(&plan_copy);
+		last_segment->depth.mm += 1000;
+		last_segment->next->depth.mm += 1000;
+		plan(&plan_copy, dive, 1, deeper, &cache, true, false);
+		free_dps(&plan_copy);
+		restore_deco_state(save, false);
+
+		last_segment = cloneDiveplan(&plan_copy);
+		last_segment->depth.mm -= 1000;
+		last_segment->next->depth.mm -= 1000;
+		plan(&plan_copy, dive, 1, shallower, &cache, true, false);
+		free_dps(&plan_copy);
+		restore_deco_state(save, false);
+
+		last_segment = cloneDiveplan(&plan_copy);
+		last_segment->next->time += 60;
+		plan(&plan_copy, dive, 1, longer, &cache, true, false);
+		free_dps(&plan_copy);
+		restore_deco_state(save, false);
+
+		last_segment = cloneDiveplan(&plan_copy);
+		last_segment->next->time -= 60;
+		plan(&plan_copy, dive, 1, shorter, &cache, true, false);
+		free_dps(&plan_copy);
+		restore_deco_state(save, false);
+#ifdef SHOWSTOPVARIATIONS
+		printf("\n\n");
+#endif
+
+		QString notes(displayed_dive.notes);
+		free(displayed_dive.notes);
+
+		char buf[200];
+		sprintf(buf, "+ %d:%02d /m + %d:%02d /min", FRACTION(analyzeVariations(shallower, original, deeper, "m"),60),
+			FRACTION(analyzeVariations(shorter, original, longer, "min"), 60));
+
+		displayed_dive.notes = strdup(notes.replace("VARIATIONS", QString(buf)).toUtf8().data());
+	}
+	setRecalc(oldRecalc);
+}
+
 void DivePlannerPointsModel::createPlan(bool replanCopy)
 {
 	// Ok, so, here the diveplan creates a dive
-	char *cache = NULL;
+	struct deco_state *cache = NULL;
 	bool oldRecalc = setRecalc(false);
 	removeDeco();
 	createTemporaryPlan();
 	setRecalc(oldRecalc);
 
 	//TODO: C-based function here?
-	plan(&diveplan, &cache, isPlanner(), true);
+	struct decostop stoptable[60];
+	plan(&diveplan, &displayed_dive, DECOTIMESTEP, stoptable, &cache, isPlanner(), true);
 	free(cache);
 	if (!current_dive || displayed_dive.id != current_dive->id) {
 		// we were planning a new dive, not re-planning an existing on
@@ -916,6 +1110,7 @@ void DivePlannerPointsModel::createPlan(bool replanCopy)
 		copy_dive(&displayed_dive, current_dive);
 	}
 	mark_divelist_changed(true);
+	sort_table(&dive_table);
 
 	// Remove and clean the diveplan, so we don't delete
 	// the dive by mistake.
